from regtester.RegExTest import RegExTest
from regtester.RegExTestItem import RegExTestItem
from regtester.TestItemResult import TestItemResult
from regtester.TestResponse import TestResponse
import os
import argparse
import sys
import importlib
from typing import List
from colorama import init, deinit
from colorama import Fore, Back, Style
class RegExTester:

    def __init__(self,path:str):
        init() #useful for window
        self.tests:RegExTest = []
        self.current_regex_test : RegExTest = None
        self.find_regex_tests(path)
        results = self.all_regex_test()
        self.show_results(results)
    def find_regex_tests(self,path:str) -> List[RegExTest]:
        # folder with the tests classes has to contain a __init__.py file
        if(os.path.isfile(path)):
            class_name = path.replace(".py","")
            module = importlib.import_module(class_name)
            class_instance =  getattr(module, class_name)()
            self.tests.append(class_instance)
        elif(os.path.isdir(path)):
            sys.path.append(path)
            for filename in os.listdir(path):
                if filename.endswith("Test.py"): 
                    class_name = filename.replace(".py","")
                    class_module = eval(f"importlib.import_module('{class_name}')")
                    class_instance =  getattr(class_module, class_name)()
                    self.tests.append(class_instance)
            self.info(f"{len(self.tests)} Test classes were found")
        else:
            raise ValueError('The path is neither a valid folder or a valid file')

    def test_one(self,item:RegExTestItem) -> TestItemResult:
        result = self.current_regex_test.exec_regex(
            self.current_regex_test.compiled_regex,
            item.text
        )
        return TestItemResult(
            text = item.text,
            expected_result = item.expected_result,
            actual_result = result,
            success = result == item.expected_result
        )

    def test_all_examples(self,reg_test:RegExTest) -> TestResponse:
        self.current_regex_test = reg_test
        items:List[RegExTestItem] = reg_test.get_tests()
        test_item_results = [self.test_one(item) for item in items]
        successes = [item for item in test_item_results if item.success]
        failures = [item for item in test_item_results if not item.success]
        return TestResponse(
            all_good=len(failures) == 0,
            regex_name = self.current_regex_test.get_regex_name(),
            success_count = len(successes),
            failure_count = len(failures),
            total = len(items),
            successes=successes,
            failures=failures,
        )

    def all_regex_test(self) -> List[TestResponse]:
            return [self.test_all_examples(reg_test) for reg_test in self.tests]

    @staticmethod
    def ok_message(text:str):
        print(Style.BRIGHT + Back.GREEN + Fore.WHITE)
        print(text)
        print(Style.RESET_ALL)

    @staticmethod
    def error_message(text:str):
        print(Style.BRIGHT + Back.RED + Fore.WHITE)
        print(text)
        print(Style.RESET_ALL)

    @staticmethod
    def info(text:str):
        print(Style.BRIGHT + Back.CYAN + Fore.WHITE)
        print(text)
        print(Style.RESET_ALL)
    def show_results(self,results:List[TestResponse]) -> None:
        for test_response in results:
            print("...........................................................")
            if(test_response.all_good):
                self.ok_message(f"{test_response.regex_name} => OK  ({test_response.success_count}/{test_response.total})")
            else:
                self.error_message(f"{test_response.regex_name} => Failures  ({test_response.failure_count}/{test_response.total})")
                for item in test_response.failures:
                    self.error_message(f"text => {item.text}\nexpected => {item.expected_result} \ngot => {item.actual_result}")

        all_test = sum([test_response.total for test_response in results])
        test_passed = sum([test_response.success_count for test_response in results])
        test_failed = sum([test_response.failure_count for test_response in results])
        self.info(f"Total tests : {all_test}, Successes  {test_passed}, Failures :  {test_failed}")
        deinit()
if __name__=="__main__":
    parser = argparse.ArgumentParser(description = "A RegEx Tester") 
    parser.add_argument("test_path", nargs = '1', metavar = "path", type = str,  
                     help = "the path is used toe xecute tests, it can be a folder containing a __init__.py file or a file")
    args = parser.parse_args() 
    if(args.test_path):
        regex_tester = RegExTester(args.test_path)



from regtester.RegExTestItem import RegExTestItem
from regtester.RegExTest import RegExTest
import regex
import re
from re import RegexFlag
from typing import Pattern, List

class ArtNumberTest(RegExTest):

    def get_regex(self) -> str:
        return r"""
        # USE THIS WITH r WHEN CREATING THE STRING IN PYTHON
        ^\s*Article\s((?:p|P)remier)
        |Art\.\s?(\d+)
        |^\s*Article\s(\d+)|^\s*Article\s([A-Za-z]+\.\d+)
        """

    def get_regex_flags(self)-> RegexFlag:
        return re.DOTALL|re.M|re.VERBOSE
    
    def get_regex_name(self)-> str:
        return "extract_article_number"
        
    def get_tests(self)-> List[RegExTestItem]:
        return [
            RegExTestItem(
                text= "Article 2: le projet TELEDAC est doté des organes suivants:\npilotage, un comité technique et une équipe de projet.\n\nun comité de\n\n2\n",
                expected_result="2"
            ),
             RegExTestItem(
                text= "Article premier: le projet TELEDAC est doté des organes suivants:\npilotage, un comité technique et une équipe de projet.\n\nun comité de\n\n2\n",
                expected_result="1"
            )
        ]

    def exec_regex(self, regex:Pattern, text:str) -> any:
        article_numbers = regex.findall(text)
        assert len(article_numbers) > 0
        article_numbers = article_numbers[0] # get the one and only tuple
        article_number =  list(filter(lambda x:x!="",article_numbers))[0] # only one element in the tuple can be non empty
        return "1" if article_number=="premier" else article_number





from regtester.RegExTestItem import RegExTestItem
from regtester.RegExTester import RegExTester
from regtester.RegExTest import RegExTest
from regtester.TestItemResult import TestItemResult
from regtester.TestResponse import TestResponse



#!/home/thierno/miniconda3/envs/regex_tester/bin/python

# importing required modules 
import argparse 
  
# create a parser object 
parser = argparse.ArgumentParser(description = "An addition program") 
  
# add argument 
parser.add_argument("add", nargs = '*', metavar = "num", type = int,  
                     help = "All the numbers separated by spaces will be added.") 
  
# parse the arguments from standard input 
args = parser.parse_args() 
  
# check if add argument has any input data. 
# If it has, then print sum of the given numbers 
if len(args.add) != 0: 
    print(sum(args.add)) 



from regtester.RegExTestItem import RegExTestItem
from regtester.RegExTest import RegExTest
import regex
import re
from re import RegexFlag
from typing import Pattern, List


class LawTypeTest(RegExTest):

    def get_regex(self) -> str:
        return r"""
        # USE THIS WITH r WHEN CREATING THE STRING IN PYTHON
        ^.{0,100}(d(?:é|e)cret|arr(?:e|ê)t(?:e|é)\s*minist(?:e|é)riel|loi)\s+n°\s*(\d{1,4})(?:\s*)(\S+)?(-\d{1,4}|\d{1,4}\s*(\S{1,3}-\S{1,4}-\S{4}\s*)?)?\s*(?:en\s+date)?\s*du\s+(?:(?:lundi|mardi|mercredi|jeudi|vendredi|samedi|dimanche)?\s+(\d{1,2}\s+(?:janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre|\d{1,2})\s+\d{2,4}))\s+(?:(?:portant|relatif|modifiant|fixant)(?:code)?\s+(?:de)?\s*(.*?)\.)?
        """

    def get_regex_flags(self)-> RegexFlag:
        return re.DOTALL|re.M|re.VERBOSE
    
    def get_regex_name(self)-> str:
        return "extract_doc_type"
        
    def get_tests(self)-> List[RegExTestItem]:
        return [
            RegExTestItem(
                text= """
                LOI n° 2009-24 du 8 juillet 2009 LOI n° 2009-24 du 8 juillet 2009 portant Code de l’Assainissement.
                 EXPOSE DE MOTIFS L’Etat du Sénégal s’est engagé, depuis 2005, dans le processus 
                 d’attente des Objectifs du Millénaire pour le Développement (OMD) qui consistent
                  à réduire de moitié la pauvreté dans les pays en développement, non pas en termes de 
                """,
                expected_result={
                    'doc_type': "loi", 'date': "8 juillet 2009", 
                    'obj': "Code de l’Assainissement", 'number': "2009-24"
                    }
            ),
             RegExTestItem(
                text= """
                Arrêté ministériel n° 9523 MEF-CGCPE en date du 17 octobre 2007 portant collecte d’informations comptables.
                 Article premier. - La Cellule de Gestion et de Contrôle du Portefeuille de l’Etat est 
                 habilité à collecter auprès des entreprises du secteur parapublic et des sociétés
                  anonymes à participation publique minoritaire, les documents suivants : 
                """,
                expected_result={
                    'doc_type': "Arrêté ministériel", 'date': "17 octobre 2007", 
                    'obj': "collecte d’informations comptables", 'number': "9523 MEF-CGCPE"
                    }            )
        ]

    def exec_regex(self, regex:Pattern, text:str)-> any:
        results = regex.findall(text)
        if len(results) <= 0:
            return None
        else:
            match = results[0]
            doc_type = match[0].lower()
            number = match[1] + " " + match[2]
            date = match[5].lower()
            obj = match[6].lower()
            return {'doc_type': doc_type, 'date': date, 'obj': obj, 'number': number}



from dataclasses import dataclass
from typing import List

from regtester.TestItemResult  import TestItemResult
@dataclass
class TestResponse:
    all_good: bool
    regex_name: str
    success_count:int
    failure_count:int
    total:int
    successes : List[TestItemResult] 
    failures : List[TestItemResult]

if __name__ == "__main__":
    x = TestResponse(
        status=True,
        regex_name="lol",
        success_count=12,
        failure_count=2,
        successes = [TestItemResult("text","expected","expected")],
        failures = [TestItemResult("text","expected","got")]
    )
    print(x)


from dataclasses import dataclass

@dataclass
class TestItemResult:
    text: str
    expected_result: any
    actual_result:any
    success:bool

if __name__ == "__main__":
    x = TestItemResult("love","pol","lop",False)
    print(x)